/*
*********************************************
*											*
*		Astro TIME functions				*
*											*
*********************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "atime.h"

/*
=========================================
			reap second table
=========================================
*/
REAPSEC tDAT[] = {
	{2457754.5, 36.0/DAYSEC},	// 2017 1 1
	{2457204.5, 35.0/DAYSEC},	// 2015 7 1
	{2456109.5, 34.0/DAYSEC},	// 2012 7 1
	{2454832.5, 33.0/DAYSEC},	// 2009 1 1
	{2453736.5, 33.0/DAYSEC},	// 2006 1 1
	{0.0, 32.0/DAYSEC}
};

/*
#############################################
#											#
#			Julian day functions			#
#											#
#############################################
*/
/*
=============================================
=			date to julian					=
=============================================
*/
double julian(int nYear, int nMonth, double dDay)
{
	double da, db;
	double dJD;

	if (nMonth < 3){
		nMonth += 12;
		nYear--;
	}

	// Calc. body
	da = floor(nYear/100.0);
	db = 2.0 - da + floor(da / 4.0);
	dJD = floor(365.25 * nYear) + floor(30.6001 * (nMonth + 1)) + dDay + db + 1720994.5;

	if (dJD < 2299160.5){
		dJD -= db;
	}

	return dJD;
}

/*
=============================================
=			julian to date					=
=============================================
*/
void juian_date(double dJD, int *nYear, int *nMonth, double *dDay)
{
	int nY, nM;
	double da, db, dd;
	double dJDs = dJD + 0.5;
	double dJDi = floor(dJDs);
	double dJDf = dJDs - dJDi;

	if (dJDi > 2299160.0){
		da = floor((dJDi - 1867216.25) / 36524.25);
		dJDi += (1.0 + da - floor(da / 4.0));
	}

	db = dJDi + 1524.0;
	nY = (int)(floor((db - 122.1) / 365.25));
	dd = db - floor(nY * 365.25);
	nM = (int)(floor(dd / 30.6001));

	// Write Day
	*dDay = dd - floor(nM * 30.6001) + dJDf;

	// Write Month
	if (nM < 14){
		*nMonth = nM - 1;
	}else{
		*nMonth = nM - 13;
	}

	// Write Year
	if (*nMonth > 2){
		*nYear = nY - 4716;
	}else{
		*nYear = nY - 4715;
	}
}

/*
=============================================
=			julian to week					=
=============================================
*/
int jd_week(double dJD)
{
	int nJD = (int)floor(dJD + 0.5);
	int nWeek = nJD % 7;
	if (nWeek < 0){
		nWeek += 7;
	}

	return nWeek;
}

/*
#############################################
#											#
#			UTC <-> TAI functions			#
#											#
#############################################
*/
/*
=============================================
=			UTC to TAI						=
=											=
=entry: DATETIME *pUTC = UTC date & time	=
=		double *pdJD_tai = TAI store adr.	=
=											=
=exit:	return double TAI-UTC sec. unit		=
=		pdJD_tai in TAI						=
=============================================
*/
double utc_tai(DATETIME *pUTC, double *pdJD_tai)
{
	double dJD;
	double ddAT;
	double dTimeDay;
	REAPSEC *ptbl;

	/*	make date to julian	& time_day */
	dJD = julian(pUTC->nYear, pUTC->nMonth, (double)pUTC->nDay);
	dTimeDay = pUTC->nHour / 24.0 + pUTC->nMin / 1440.0 + pUTC->dSec / DAYSEC;

	/* get delta AT	*/
	ptbl = tDAT;
	while( dJD < ptbl->dJD){
		ptbl++;
	}
	ddAT = ptbl->dTime;

	/* add deltaAT */
	*pdJD_tai = dJD + dTimeDay + ddAT;
	return (ddAT * DAYSEC);
}

/*
=============================================
=			TAI to UTC						=
=											=
=entry: double *pdJD_tai = TAI data			=
=		DATETIME *pUTC = UTC store adr		=
=											=
=exit:	return double TAI-UTC sec. unit		=
=		pUTC in UTC date & time				=
=============================================
*/
double tai_utc(double dJD_TAI, DATETIME *pUTC)
{
	double dJD;
	double dDay;
	double ddAT;
	REAPSEC *ptbl;
	int n60sec = 0;

	/* get deltaAT	*/
	ptbl = tDAT;
	while( dJD_TAI < (ptbl->dJD + ptbl->dTime)){
		ptbl++;
	}
	ddAT = ptbl->dTime;

	/* TAI to UTC */
	dJD = dJD_TAI - ddAT;

	/* check UTC 60s */
	if (ptbl != tDAT){
		ptbl--;
		if (dJD >= ptbl->dJD){
			/* 60s unit */
			dJD -= 1.0 / DAYSEC;
			n60sec = 1;
		}
	}

	/*	julian to date */
	juian_date(dJD, &pUTC->nYear, &pUTC->nMonth, &dDay);
	pUTC->nDay = (int)floor(dDay);
	dayfrac_time(dDay - pUTC->nDay, &pUTC->nHour, &pUTC->nMin, &pUTC->dSec);

	/* make 60s */
	pUTC->dSec += n60sec;

	return (ddAT * DAYSEC);
}

/*
#############################################
#											#
#		other common functions				#
#											#
#############################################
*/
/*
=============================================
=		daytime to time structer			=
=============================================
*/
void dayfrac_time(double dDayFrac, int *pnHour, int *pnMin, double *pdSec)
{
	double dTemp;

	dTemp = dDayFrac * DAYHOUR;
	*pnHour = (int)floor(dTemp);
	dTemp -= *pnHour;

	dTemp *= 60.0;
	*pnMin = (int)floor(dTemp);
	dTemp -= *pnMin;

	*pdSec = dTemp * 60.0;
}

